;;; GitCube --- The Git cube for FG42 -*- lexical-binding: t; -*-
;;
;; Copyright (c) 2010-2024 Sameer Rahmani & Contributors
;;
;; Author: Sameer Rahmani <lxsameer@gnu.org>
;; URL: https://devheroes.codes/FG42/FG42
;; Version: 4.0.0
;;
;; This program is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation, either version 3 of the License, or
;; (at your option) any later version.
;;
;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;
;;; Commentary:
;; Cubes are the building blocks of any `FG42' editor.  Each `cube' is a
;; unit which defines different abilities in a deterministic and idempotent
;; way.  Cubes are composable and a composition of cubes creates an editor.
;;
;;; Code:
(require 'fpkg)
(require 'fg42/cube)


(defcube fg42/diff-hl-cube
  "This cube highlights changes to the current buffer in respect to the
VC status of the file. For example if you're using *git* in your project
and you've made some changes to the current buffer that are not commited
yet it will highlihgt them for you.

For more info check out [[https://github.com/dgutov/diff-hl][diff-hl]] 's page."
  :title "Diff Highlight Cube"
  :flag diff-hl
  :flag-default t

  (fpkg/use diff-hl
    :init
    (progn
      (require 'diff-hl)
      (when-flag git
        (add-hook 'magit-pre-refresh-hook 'diff-hl-magit-pre-refresh)
        (add-hook 'magit-post-refresh-hook 'diff-hl-magit-post-refresh))
      (global-diff-hl-mode))))


(defcube fg42/git-cube
  "This cube integrates *git* into *FG42*."
  :title "Git Cube"
  :flag git
  :flag-default t

  (fpkg/use magit
    :bind (("C-x g" . magit-status)))

  (fpkg/use magit-todos
    :after magit
    :init (require 'magit-todos))

  (fg42/diff-hl-cube))


(provide 'fg42/cubes/git)
;;; git.el ends here
