;;; Terraform --- The terraform cube for FG42 -*- lexical-binding: t; -*-
;;
;; Copyright (c) 2010-2024 Sameer Rahmani & Contributors
;;
;; Author: Sameer Rahmani <lxsameer@gnu.org>
;; URL: https://devheroes.codes/FG42/FG42
;; Version: 4.0.0
;;
;; This program is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation, either version 3 of the License, or
;; (at your option) any later version.
;;
;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;
;;; Commentary:
;; Cubes are the building blocks of any `FG42' editor.  Each `cube' is a
;; unit which defines different abilities in a deterministic and idempotent
;; way.  Cubes are composable and a composition of cubes creates an editor.
;;
;;; Code:
(require 'fpkg)
(require 'fg42/cube)


(defcube fg42/terraform-cube
  "terraform cube"
  :title "fg42/cubes/fg42/terraform-cube.org"
  :flag terraform
  :flag-default t

  ;; https://github.com/juliosueiras/terraform-lsp is required
  (let ((terraform-lsp (or (plist-get fg42/terraform-cube-params :terraform-lsp-path) "terraform-lsp"))
        (lsp-flags (or (plist-get fg42/terraform-cube-params :terraform-lsp-flags) "-enable-log-file")))
    (fpkg/use terraform-mode
      :config
      (progn
        (when-flag lsp
          (require 'lsp)
          (add-to-list 'lsp-language-id-configuration '(terraform-mode . "terraform"))

          (lsp-register-client
           (make-lsp-client :new-connection (lsp-stdio-connection (list terraform-lsp lsp-flags))
                            :major-modes '(terraform-mode)
                            :server-id 'terraform-ls))

          (add-hook 'terraform-mode-hook #'lsp))))))


(provide 'fg42/cubes/terraform)
;;; terraform.el ends here
