;;; fpkg --- a simple package manager for FG42                     -*- lexical-binding: t; -*-
;;
;; Copyright (C) 2010-2024  Sameer Rahmani <lxsameer@gnu.org>
;;
;; Author: Sameer Rahmani <lxsameer@gnu.org>
;; Keywords: lisp fg42 IDE package manager
;; Version: 1.0.0
;;
;; This program is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation, either version 3 of the License, or
;; (at your option) any later version.
;;
;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;
;;; Commentary:
;;
;; Simple package manager for FG42
;;
;;; Code:
(require 'map)

(defvar package-names ())

(eval-when-compile
  (defvar fg42/disabled-features '())
  (require 'use-package))


(defun inject-params (args)
  "Inject required `use-package' params to ARGS if the key is missing."
  ;; (if (member :defer args)
  ;;     (append '(:ensure nil) args)
  (append args '(:ensure nil)))


(defmacro fpkg/use (pkg &rest details)
  "Install the given package DETAILS PKG via `use-package' and straight."
  (declare (indent defun))
  (if (and (listp details) (< 0 (length details)))
      (let ((p (inject-params details)))
        `(progn
           (use-package ,pkg ,@p)))
    `(progn
       (use-package ,pkg :defer t :ensure nil))))


(defmacro use! (pkg docs &rest details)
  "Loading the given package DETAILS PKG via `use-package'.

DOCS is the documentation of the package."
  (declare (indent defun) (doc-string 2))

  (when (not (stringp docs))
    (error "Missing docstring for '%s' package" pkg))

  (let ((disabled (or (member pkg fg42/disabled-features) nil)))
    (when (not disabled)
      `(use-package ,pkg ,@details))))


(provide 'fpkg)
;;; fpkg.el ends here
